import { BEFORE_REQUEST, AFTER_REQUEST, INTERRUPTED } from './constants.js';
import { validateMiddleware } from './validateMiddleware.js';

export { INTERRUPTED };

export class Middleware {
  constructor() {
    this._index = 0;
    this._middlewares = [];
    this._requests = {};

    this.executePreRequest = this.executePreRequest.bind(this);
    this.executePostRequest = this.executePostRequest.bind(this);
    this._proceed = this._proceed.bind(this);
    this._quit = this._quit.bind(this);
  }

  getAll() {
    return this._middlewares;
  }

  use(middleware) {
    this._middlewares.push(validateMiddleware(middleware));
  }

  executePreRequest(requestId) {
    return arg => this._exec(BEFORE_REQUEST, requestId, arg);
  }

  executePostRequest(requestId) {
    return arg => this._exec(AFTER_REQUEST, requestId, arg);
  }

  _exec(stage, requestId, arg) {
    this._requests[requestId] = this._requests[requestId] || {};
    this._requests[requestId].stage = stage;

    this._executeCurrentMiddleware(requestId);

    return new Promise((resolve) => {
      this._requests[requestId].resolver = (signal) => {
        resolve(signal || arg);
      };
    });
  }

  _executeCurrentMiddleware(requestId) {
    const currentMiddleware = this._middlewares[this._index];
    const { stage } = this._requests[requestId];

    if (!currentMiddleware || !currentMiddleware[stage]) {
      this._proceed(requestId);
      return undefined;
    }

    const currentCallback = currentMiddleware[stage];

    const signal = stage === BEFORE_REQUEST
      ? INTERRUPTED
      : undefined;

    currentCallback({
      proceed: () => this._proceed(requestId),
      quit: () => this._quit(requestId, signal),
    });
    return undefined;
  }

  _proceed(requestId) {
    if (this._index + 1 < this._middlewares.length) {
      this._index += 1;
      this._executeCurrentMiddleware(requestId);
      return undefined;
    }

    this._quit(requestId);
    return undefined;
  }

  _quit(requestId, signal) {
    Promise.resolve()
      .then(() => {
        this._requests[requestId].resolver(signal);
        this._requests[requestId] = undefined;
        this._index = 0;
      });
  }
}

export const middleware = new Middleware();
