import { makeRequest } from './makeRequest.js';
import { uniqId } from './uniqId.js';

import { get as getBaseUrl } from '../services/baseUrl.js';
import { get as getHeader } from '../services/headers.js';
import { validateRequiredParams } from './validateRequiredParams.js';
import { validateBaseUrl } from './validateBaseUrl.js';
import { validatePayload } from './validatePayload.js';
import { middleware, INTERRUPTED } from '../services/middleware/middleware.js';

import { GET, POST, PUT, PATCH } from '../constants/constants.js';

export const request = (
  urlMaker,
  method = GET,
  requestInterface = {},
  requestConfig = {},
  fetchMaker = makeRequest,
) => (...userArgs) => {
  try {
    const [urlParams = {}, payload = {}] =
      userArgs.filter(arg => typeof arg === 'object');

    const userBaseUrl =
      userArgs.find(arg => typeof arg === 'string') || getBaseUrl();

    const resource = validateBaseUrl(userBaseUrl);

    const url = (typeof urlMaker === 'function')
      ? urlMaker(urlParams)
      : urlMaker || '';

    const validUrl = validateRequiredParams(`${resource}${url}`);

    let requestParams = {
      method,
      headers: getHeader(),
    };

    if (Object.keys(payload).length > 0) {
      requestParams = {
        ...requestParams,
        body: validatePayload(requestInterface, payload, method),
      };
    }

    const requestId = uniqId();

    const executeRequest = () =>
      fetchMaker(validUrl, requestParams, requestConfig);

    return Promise.resolve()
      .then(middleware.executePreRequest(requestId))
      .then(signal => (signal !== INTERRUPTED
        ? executeRequest().then(middleware.executePostRequest(requestId))
        : undefined));
  } catch (err) {
    return Promise.reject(err);
  }
};

export const getRequest = (urlMaker, ...args) =>
  request(urlMaker, GET, undefined, ...args);

export const postRequest = (urlMaker, requestInterface, ...args) =>
  request(urlMaker, POST, requestInterface, ...args);

export const putRequest = (urlMaker, requestInterface, ...args) =>
  request(urlMaker, PUT, requestInterface, ...args);

export const patchRequest = (urlMaker, requestInterface, ...args) =>
  request(urlMaker, PATCH, requestInterface, ...args);
