import sinon from 'sinon';
import { get as getBaseUrl } from '../services/baseUrl.js';
import { GET } from '../constants/constants.js';

import {
  request,
  getRequest,
  postRequest,
  putRequest,
  patchRequest,
} from './request.js';

const mockUrlMaker = params => `v0/transactions/brands&${params.affiliationCode}`;

const mockUrl = 'v0/transactions/brands&111111111';

const mockRequestParams = { affiliationCode: '111111111' };

const defaultBaseUrl = getBaseUrl();

const defaultRequestParams = {
  method: GET,
  headers: {},
};

let fetchMaker;

describe('Request Helper', () => {
  beforeEach(() => {
    fetchMaker = sinon.stub().returns(Promise.resolve());
  });

  afterEach(() => {
    fetchMaker = undefined;
  });

  it('Should call fetchMaker without any arguments.', async () => {
    await request(undefined, undefined, undefined, undefined, fetchMaker)();

    expect(fetchMaker).to.have.been
      .calledWith(defaultBaseUrl, defaultRequestParams);
  });

  it('Should call fetchMaker passing only an url string.', async () => {
    await request(mockUrl, undefined, undefined, undefined, fetchMaker)();

    expect(fetchMaker).to.have.been
      .calledWith(`${defaultBaseUrl}${mockUrl}`, defaultRequestParams);
  });

  it('Should call fetchMaker passing only request parameters.', async () => {
    await request(
      undefined,
      undefined,
      undefined,
      undefined,
      fetchMaker,
    )(mockRequestParams);

    expect(fetchMaker).to.have.been
      .calledWith(defaultBaseUrl, defaultRequestParams);
  });

  it('Should call fetchMaker an urlMaker function and ' +
    'request parameters.', async () => {
    await request(
      mockUrlMaker,
      undefined,
      undefined,
      undefined,
      fetchMaker,
    )(mockRequestParams);

    expect(fetchMaker).to.have.been
      .calledWith(`${defaultBaseUrl}${mockUrl}`, defaultRequestParams);
  });

  it('Should call fetchMaker with a custom base url and ' +
    'an urlMaker function.', async () => {
    await request(mockUrlMaker, undefined, undefined, undefined, fetchMaker)(
      'test/', mockRequestParams);

    expect(fetchMaker).to.have.been
      .calledWith(`test/${mockUrl}`, defaultRequestParams);
  });

  it('Should call fetchMaker without a custom base url and ' +
    'an urlMaker function.', async () => {
    await request(
      undefined,
      undefined,
      undefined,
      undefined,
      fetchMaker,
    )('test/', mockRequestParams);

    expect(fetchMaker).to.have.been
      .calledWith('test/', defaultRequestParams);
  });

  it('Should call fetchMaker with a custom baseUrl, ' +
    'an urlMaker function and a payload.', async () => {
    const mockPayload = {
      data: 'test',
    };

    const mockModel = {
      data: String,
    };

    const mockRequestParam = {
      ...defaultRequestParams,
      body: { data: 'test' },
    };

    await request(
      mockUrlMaker,
      undefined,
      mockModel,
      undefined,
      fetchMaker,
    )(mockRequestParams, mockPayload);

    expect(fetchMaker).to.have.been
      .calledWith(`${defaultBaseUrl}${mockUrl}`, mockRequestParam);
  });

  it('Should throw inside the promise if any of the validations fail.',
    (done) => {
      request()('bogus-base-url')
        .catch((err) => {
          expect(err).to.be.an('error');
          done();
        });
    });
});

describe('getRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(getRequest()).to.be.a('function');
  });
});

describe('postRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(postRequest()).to.be.a('function');
  });
});

describe('putRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(putRequest()).to.be.a('function');
  });
});

describe('patchRequest', () => {
  it('Should be a shortcut for request.', () => {
    expect(patchRequest()).to.be.a('function');
  });
});
